@@ -4,7 +4,8 @@ import random
 import re
 
 from config import CONFIG as cfg
-from db import _key, msetbit, sequential_id
+from db import _key, msetbit, sequential_id, first_key_with_bit_set
+from math import log
 
 # This is pretty restrictive, but we can always relax it later.
 VALID_EXPERIMENT_ALTERNATIVE_RE = re.compile(r"^[a-z0-9][a-z0-9\-_ ]*$", re.I)
@@ -16,33 +17,6 @@ class Client(object):
     def __init__(self, client_id, redis_conn):
         self.redis = redis_conn
         self.client_id = client_id
-        self._sequential_id = None
-
-    @property
-    def sequential_id(self):
-        if self._sequential_id is None:
-            self._sequential_id = sequential_id(
-                'internal_user_ids',
-                self.client_id)
-        return self._sequential_id
-
-
-class ExperimentCollection(object):
-
-    def __init__(self, redis_conn):
-        self.redis = redis_conn
-        self.experiments = []
-
-    def __iter__(self):
-        self.experiments = []
-        for exp_key in self.redis.smembers(_key('experiments')):
-            self.experiments.append(exp_key)
-
-        return self
-
-    def __next__(self):
-        for i in self.experiments:
-            yield Experiment.find(i, self.redis)
 
 
 class Experiment(object):
@@ -56,6 +30,9 @@ class Experiment(object):
         self.random_sample = RANDOM_SAMPLE
         self.alternatives = self.initialize_alternatives(alternatives)
         self._version = version
+        # False here is a sentinal value for "not looked up yet"
+        self._winner = False
+        self._sequential_ids = dict()
 
     def __repr__(self):
         return '<Experiment: {0} (version: {1})>'.format(self.name, self.version())
@@ -64,7 +41,14 @@ class Experiment(object):
         objectified = {
             'name': self.name,
             'period': period,
-            'alternatives': []
+            'alternatives': [],
+            'created_at': self.created_at(),
+            'total_participants': self.total_participants(),
+            'total_conversions': self.total_conversions(),
+            'description': self.get_description(),
+            'has_winner': self.winner is not None,
+            'is_archived': self.is_archived(),
+            'version': self.version()
         }
 
         for alternative in self.alternatives:
@@ -73,6 +57,32 @@ class Experiment(object):
 
         return objectified
 
+    def csvify(self):
+        import cStringIO as StringIO
+        import csv
+        csvfile = StringIO.StringIO()
+        writer = csv.writer(csvfile)
+        writer.writerow(['Alternative Details'])
+        writer.writerow(['date', 'alternative', 'participants', 'conversions'])
+        obj = self.objectify_by_period('day')
+        for alt in obj['alternatives']:
+            for datum in alt['data']:
+                writer.writerow([datum['date'], alt['name'], datum['participants'], datum['conversions']])
+        writer.writerow([])
+
+        writer.writerow(['"{0}" Summary'.format(obj['name'])])
+        writer.writerow(['total participants', 'total_conversions', 'has_winner', 'description'])
+        writer.writerow([obj['total_participants'], obj['total_conversions'], obj['has_winner'], obj['description']])
+
+        writer.writerow([])
+        writer.writerow(['Alternative Summary'])
+
+        writer.writerow(['name', 'participant_count', 'completed_count'])
+        for alt in obj['alternatives']:
+            writer.writerow([alt['name'], alt['participant_count'], alt['completed_count']])
+
+        return csvfile.getvalue()
+
     def initialize_alternatives(self, alternatives):
         for alternative_name in alternatives:
             if not Alternative.is_valid(alternative_name):
@@ -83,10 +93,10 @@ class Experiment(object):
     def save(self):
         pipe = self.redis.pipeline()
         if self.is_new_record():
-            pipe.sadd(_key('experiments'), self.name)
+            pipe.sadd(_key('e'), self.name)
 
             # Set version to zero
-            pipe.set(_key("experiments:{0}".format(self.name)), 0)
+            pipe.set(_key("e:{0}".format(self.name)), 0)
 
         pipe.hset(self.key(), 'created_at', datetime.now())
         # reverse here and use lpush to keep consistent with using lrange
@@ -95,6 +105,7 @@ class Experiment(object):
 
         pipe.execute()
 
+    @property
     def control(self):
         return self.alternatives[0]
 
@@ -105,10 +116,10 @@ class Experiment(object):
         return [alt.name for alt in self.alternatives]
 
     def is_new_record(self):
-        return not self.redis.sismember(_key("experiments"), self.name)
+        return not self.redis.sismember(_key("e"), self.name)
 
     def total_participants(self):
-        key = _key("participations:{0}:_all:all".format(self.rawkey()))
+        key = _key("p:{0}:_all:all".format(self.rawkey()))
         return self.redis.bitcount(key)
 
     def participants_by_day(self):
@@ -121,7 +132,7 @@ class Experiment(object):
         return self._get_stats('participations', 'years')
 
     def total_conversions(self):
-        key = _key("conversions:{0}:_all:users:all".format(self.rawkey()))
+        key = _key("c:{0}:_all:users:all".format(self.rawkey()))
         return self.redis.bitcount(key)
 
     def conversions_by_day(self):
@@ -134,11 +145,15 @@ class Experiment(object):
         return self._get_stats('conversions', 'years')
 
     def _get_stats(self, stat_type, stat_range):
-        if stat_type not in ['participations', 'conversions']:
-            raise ValueError
+        if stat_type == 'participations':
+            stat_type = 'p'
+        elif stat_type == 'conversions':
+            stat_type = 'c'
+        else:
+            raise ValueError("Unrecognized stat type: {0}".format(stat_type))
 
         if stat_range not in ['days', 'months', 'years']:
-            raise ValueError
+            raise ValueError("Unrecognized stat range: {0}".format(stat_range))
 
         pipe = self.redis.pipe()
 
@@ -146,7 +161,7 @@ class Experiment(object):
         search_key = _key("{0}:{1}:{2}".format(stat_type, self.rawkey(), stat_range))
         keys = self.redis.smembers(search_key)
         for k in keys:
-            mod = '' if stat_type == 'participations' else "users:"
+            mod = '' if stat_type == 'p' else "users:"
             range_key = _key("{0}:{1}:_all:{2}{3}".format(stat_type, self.rawkey(), mod, k))
             pipe.bitcount(range_key)
 
@@ -173,13 +188,13 @@ class Experiment(object):
 
     def delete(self):
         pipe = self.redis.pipeline()
-        pipe.srem(_key('experiments'), self.name)
+        pipe.srem(_key('e'), self.name)
         pipe.delete(self.key())
         pipe.delete(_key(self.rawkey()))
-        pipe.delete(_key('experiments:{0}'.format(self.name)))
+        pipe.delete(_key('e:{0}'.format(self.name)))
 
         # Consider a 'non-keys' implementation of this
-        keys = self.redis.keys('*{0}*'.format(self.rawkey()))
+        keys = self.redis.keys('*:{0}:*'.format(self.name))
         for key in keys:
             pipe.delete(key)
 
@@ -199,87 +214,79 @@ class Experiment(object):
         return range(0, (current_version + 1))
 
     def version(self):
-        if self._version is not None:
-            return self._version
-
-        version = self.redis.get(_key("experiments:{0}".format(self.name)))
-        return int(version) if version else 0
+        if self._version is None:
+            v = self.redis.get(_key("e:{0}".format(self.name)))
+            self._version = int(v) if v else 0
+        return self._version
 
     def increment_version(self):
-        self.redis.incr(_key('experiments:{0}'.format(self.name)))
+        self.redis.incr(_key('e:{0}'.format(self.name)))
+        self._version = None
 
-    def convert(self, client):
-        alternative = self.get_alternative_by_client_id(client)
-
-        if not alternative:  # TODO or has already converted?
+    def convert(self, client, dt=None):
+        alternative = self.existing_alternative(client)
+        if not alternative:
             raise ValueError('this client was not participaing')
 
-        alternative.record_conversion(client)
+        if not self.existing_conversion(client):
+            alternative.record_conversion(client, dt=dt)
 
         return alternative.name
 
+    @property
+    def winner(self):
+        if not self._winner:
+            self._winner = self.redis.get(self._winner_key)
+        return self._winner
+
     def set_winner(self, alternative_name):
         if alternative_name not in self.get_alternative_names():
             raise ValueError('this alternative is not in this experiment')
-
+        self._winner = alternative_name
         self.redis.set(self._winner_key, alternative_name)
 
-    def has_winner(self):
-        return self.redis.exists(self._winner_key)
-
-    def get_winner(self):
-        if self.has_winner():
-            return self.redis.get(self._winner_key)
-
-        return None
-
     def reset_winner(self):
+        self._winner = None
         self.redis.delete(self._winner_key)
 
     @property
     def _winner_key(self):
         return "{0}:winner".format(self.key())
 
-    def get_alternative(self, client):
-        if self.is_archived():
-            return self.control()
+    def sequential_id(self, client):
+        """Return the sequential id for this test for the passed in client"""
+        if client.client_id not in self._sequential_ids:
+            id_ = sequential_id("e:{0}:users".format(self.rawkey()), client.client_id)
+            self._sequential_ids[client.client_id] = id_
+        return self._sequential_ids[client.client_id]
 
-        chosen_alternative = self.get_alternative_by_client_id(client)
+    def get_alternative(self, client, dt=None):
+        if self.is_archived():
+            return self.control
+
+        chosen_alternative = self.existing_alternative(client)
         if not chosen_alternative:
             chosen_alternative = self.choose_alternative(client=client)
-            chosen_alternative.record_participation(client)
+            chosen_alternative.record_participation(client, dt=dt)
 
         return chosen_alternative
 
-    def get_alternative_by_client_id(self, client):
-        # TODO, THIS IS SCRATCH/PROTO
-        # MOVE INTO A LUA SCRIPT
-        for alternative in self.get_alternative_names():
-            key = _key("participations:{0}:{1}:all".format(self.rawkey(), alternative))
-            if self.redis.getbit(key, client.sequential_id):
-                return Alternative(alternative, self, self.redis)
+    def existing_alternative(self, client):
+        alts = self.get_alternative_names()
+        keys = [_key("p:{0}:{1}:all".format(self.rawkey(), alt)) for alt in alts]
+        altkey = first_key_with_bit_set(keys=keys, args=[self.sequential_id(client)])
+        if altkey:
+            idx = keys.index(altkey)
+            return Alternative(alts[idx], self, self.redis)
 
         return None
 
-    def has_converted_by_client_id(self, client_id):
-        # TODO, THIS IS SCRATCH/PROTO
-        # MOVE INTO A LUA SCRIPT
-        for alternative in self.get_alternative_names():
-            key = _key("conversions:{0}:{1}:all".format(self.rawkey(), alternative))
-            if self.redis.getbit(key, client_id):
-                return True
-
-        return False
-
     def choose_alternative(self, client=None):
-        if cfg.get('disable_whiplash'):
-            return self._random_choice()
-
-        if random.random() < self.random_sample:
-            return self._random_choice()
-        else:
+        if cfg.get('enable_whiplash') and random.random() >= self.random_sample:
             return Alternative(self._whiplash(), self, self.redis)
 
+        return self._random_choice()
+
     def _random_choice(self):
         return random.choice(self.alternatives)
 
@@ -302,19 +309,35 @@ class Experiment(object):
 
         return random.betavariate(a + fairness_score, b + fairness_score)
 
+    def existing_conversion(self, client):
+        alts = self.get_alternative_names()
+        keys = [_key("c:{0}:{1}:users:all".format(self.rawkey(), alt)) for alt in alts]
+        altkey = first_key_with_bit_set(keys=keys, args=[self.sequential_id(client)])
+        if altkey:
+            idx = keys.index(altkey)
+            return Alternative(alts[idx], self, self.redis)
+
+        return None
+
     def rawkey(self):
         return "{0}/{1}".format(self.name, self.version())
 
     def key(self):
-        key = "experiments:{0}".format(self.rawkey())
+        key = "e:{0}".format(self.rawkey())
         return _key(key)
 
     @classmethod
     def find(cls, experiment_name, redis_conn, version=None):
-        if redis_conn.sismember(_key("experiments"), experiment_name):
-            return cls(experiment_name, Experiment.load_alternatives(experiment_name, redis_conn, version), redis_conn, version)
-        else:
-            raise ValueError('experiment does not exist')
+        if version is None:
+            version = redis_conn.get(_key("e:{0}".format(experiment_name)))
+            if version is None:
+                raise ValueError('experiment does not exist')
+
+        version = int(version)
+        return cls(experiment_name,
+                   Experiment.load_alternatives(experiment_name, redis_conn, version=version),
+                   redis_conn,
+                   version=version)
 
     @classmethod
     def find_or_create(cls, experiment_name, alternatives, redis_conn):
@@ -323,19 +346,12 @@ class Experiment(object):
 
         # We don't use the class method key here
         try:
-            Experiment.find(experiment_name, redis_conn)
-            # Note during refactor:
-            # We're not instanciating a new Experiment, rather than this load_alternatives hackery
+            # Get the most recent version of experiment_name
             exp = Experiment.find(experiment_name, redis_conn)
 
-            # get the existing alternatives
-            current_alternatives = exp.get_alternative_names()
-
-            # Make sure the alternative options are correct.
-            # If they are not, then we have to make a new version
-            # above `experiment` is then returned eventually
-            if sorted(current_alternatives) != sorted(alternatives):
-
+            # Make sure the alternative options are correct. If they are not,
+            # we have to make a new version.
+            if sorted(exp.get_alternative_names()) != sorted(alternatives):
                 # First, let's try to find a version of the experiment that -did- have this argument list
                 found = Experiment.find_with_alternatives(experiment_name, alternatives, redis_conn)
                 if found is not None:
@@ -357,19 +373,23 @@ class Experiment(object):
 
     @classmethod
     def find_with_alternatives(cls, experiment_name, alternatives, redis_conn):
-        versions = int(redis_conn.get(_key("experiments:{0}".format(experiment_name))))
+        versions = int(redis_conn.get(_key("e:{0}".format(experiment_name))))
         for i in reversed(range(0, versions + 1)):
-            _a_key = _key("experiments:{0}/{1}:alternatives".format(experiment_name, i))
+            _a_key = _key("e:{0}/{1}:alternatives".format(experiment_name, i))
             _alts = redis_conn.lrange(_a_key, 0, -1)
             if sorted(_alts) == sorted(alternatives):
                 return cls(experiment_name, alternatives, redis_conn, i)
 
         return None
 
+    @staticmethod
+    def all_names(redis_conn):
+        return redis_conn.smembers(_key('e'))
+
     @staticmethod
     def all(redis_conn, exclude_archived=True):
         experiments = []
-        keys = redis_conn.smembers(_key('experiments'))
+        keys = redis_conn.smembers(_key('e'))
 
         for key in keys:
             experiment = Experiment.find(key, redis_conn)
@@ -382,8 +402,8 @@ class Experiment(object):
     def load_alternatives(experiment_name, redis_conn, version=None):
         # get latest version of experiment
         if version is None:
-            version = redis_conn.get(_key("experiments:{0}".format(experiment_name)))
-        key = _key("experiments:{0}/{1}:alternatives".format(experiment_name, version))
+            version = redis_conn.get(_key("e:{0}".format(experiment_name)))
+        key = _key("e:{0}/{1}:alternatives".format(experiment_name, version))
         return redis_conn.lrange(key, 0, -1)
 
     @staticmethod
@@ -392,24 +412,6 @@ class Experiment(object):
                 VALID_EXPERIMENT_ALTERNATIVE_RE.match(experiment_name) is not None)
 
 
-class AlternativeCollection(object):
-
-    def __init__(self, redis_conn):
-        self.redis = redis_conn
-        self.alternatives = []
-
-    # def __iter__(self):
-    #     self.alternatives = []
-    #     for exp_key in self.redis.smembers(_key('experiments')):
-    #         self.experiments.append(exp_key)
-
-    #     return self
-
-    # def __next__(self):
-    #     for i in self.alternatives:
-    #         yield Experiment.find(i)
-
-
 class Alternative(object):
 
     def __init__(self, name, experiment, redis_conn):
@@ -421,7 +423,6 @@ class Alternative(object):
         return "<Alternative {0} (Experiment {1})".format(self.name, self.experiment.name)
 
     def objectify_by_period(self, period):
-
         PERIOD_TO_METHOD_MAP = {
             'day': {
                 'participants': self.participants_by_day,
@@ -453,17 +454,28 @@ class Alternative(object):
             }
             data.append(_data)
 
-        return {'name': self.name, 'data': data,
-                'control': self.is_control()}
+        objectified = {
+            'name': self.name,
+            'data': data,
+            'conversion_rate': float('%.2f' % (self.conversion_rate() * 100)),
+            'is_control': self.is_control(),
+            'is_winner': self.is_winner(),
+            'test_statistic': self.g_stat(),
+            'participant_count': self.participant_count(),
+            'completed_count': self.completed_count(),
+            'confidence_level': self.confidence_level()
+        }
+
+        return objectified
 
     def is_control(self):
-        return self.experiment.control().name == self.name
+        return self.experiment.control.name == self.name
 
     def is_winner(self):
-        return self.experiment.has_winner() and self.experiment.get_winner() == self.name
+        return self.experiment.winner == self.name
 
     def participant_count(self):
-        key = _key("participations:{0}:{1}:all".format(self.experiment.rawkey(), self.name))
+        key = _key("p:{0}:{1}:all".format(self.experiment.rawkey(), self.name))
         return self.redis.bitcount(key)
 
     def participants_by_day(self):
@@ -476,7 +488,7 @@ class Alternative(object):
         return self._get_stats('participations', 'years')
 
     def completed_count(self):
-        key = _key("conversions:{0}:{1}:users:all".format(self.experiment.rawkey(), self.name))
+        key = _key("c:{0}:{1}:users:all".format(self.experiment.rawkey(), self.name))
         return self.redis.bitcount(key)
 
     def conversions_by_day(self):
@@ -489,11 +501,15 @@ class Alternative(object):
         return self._get_stats('conversions', 'years')
 
     def _get_stats(self, stat_type, stat_range):
-        if stat_type not in ['participations', 'conversions']:
-            raise ValueError
+        if stat_type == 'participations':
+            stat_type = 'p'
+        elif stat_type == 'conversions':
+            stat_type = 'c'
+        else:
+            raise ValueError("Unrecognized stat type: {0}".format(stat_type))
 
         if stat_range not in ['days', 'months', 'years']:
-            raise ValueError
+            raise ValueError("Unrecognized stat range: {0}".format(stat_range))
 
         stats = {}
 
@@ -504,7 +520,7 @@ class Alternative(object):
 
         keys = self.redis.smembers(search_key)
         for k in keys:
-            name = self.name if stat_type == 'participations' else "{0}:users".format(self.name)
+            name = self.name if stat_type == 'p' else "{0}:users".format(self.name)
             range_key = _key("{0}:{1}:{2}:{3}".format(stat_type, exp_key, name, k))
             pipe.bitcount(range_key)
 
@@ -525,23 +541,23 @@ class Alternative(object):
 
         pipe = self.redis.pipeline()
 
-        pipe.sadd(_key("participations:{0}:years".format(experiment_key)), date.strftime('%Y'))
-        pipe.sadd(_key("participations:{0}:months".format(experiment_key)), date.strftime('%Y-%m'))
-        pipe.sadd(_key("participations:{0}:days".format(experiment_key)), date.strftime('%Y-%m-%d'))
+        pipe.sadd(_key("p:{0}:years".format(experiment_key)), date.strftime('%Y'))
+        pipe.sadd(_key("p:{0}:months".format(experiment_key)), date.strftime('%Y-%m'))
+        pipe.sadd(_key("p:{0}:days".format(experiment_key)), date.strftime('%Y-%m-%d'))
 
         pipe.execute()
 
         keys = [
-            _key("participations:{0}:_all:all".format(experiment_key)),
-            _key("participations:{0}:_all:{1}".format(experiment_key, date.strftime('%Y'))),
-            _key("participations:{0}:_all:{1}".format(experiment_key, date.strftime('%Y-%m'))),
-            _key("participations:{0}:_all:{1}".format(experiment_key, date.strftime('%Y-%m-%d'))),
-            _key("participations:{0}:{1}:all".format(experiment_key, self.name)),
-            _key("participations:{0}:{1}:{2}".format(experiment_key, self.name, date.strftime('%Y'))),
-            _key("participations:{0}:{1}:{2}".format(experiment_key, self.name, date.strftime('%Y-%m'))),
-            _key("participations:{0}:{1}:{2}".format(experiment_key, self.name, date.strftime('%Y-%m-%d'))),
+            _key("p:{0}:_all:all".format(experiment_key)),
+            _key("p:{0}:_all:{1}".format(experiment_key, date.strftime('%Y'))),
+            _key("p:{0}:_all:{1}".format(experiment_key, date.strftime('%Y-%m'))),
+            _key("p:{0}:_all:{1}".format(experiment_key, date.strftime('%Y-%m-%d'))),
+            _key("p:{0}:{1}:all".format(experiment_key, self.name)),
+            _key("p:{0}:{1}:{2}".format(experiment_key, self.name, date.strftime('%Y'))),
+            _key("p:{0}:{1}:{2}".format(experiment_key, self.name, date.strftime('%Y-%m'))),
+            _key("p:{0}:{1}:{2}".format(experiment_key, self.name, date.strftime('%Y-%m-%d'))),
         ]
-        msetbit(keys=keys, args=([client.sequential_id, 1] * len(keys)))
+        msetbit(keys=keys, args=([self.experiment.sequential_id(client), 1] * len(keys)))
 
     def record_conversion(self, client, dt=None):
         """Record a user's conversion in a test along with a given variation"""
@@ -554,23 +570,23 @@ class Alternative(object):
 
         pipe = self.redis.pipeline()
 
-        pipe.sadd(_key("conversions:{0}:years".format(experiment_key)), date.strftime('%Y'))
-        pipe.sadd(_key("conversions:{0}:months".format(experiment_key)), date.strftime('%Y-%m'))
-        pipe.sadd(_key("conversions:{0}:days".format(experiment_key)), date.strftime('%Y-%m-%d'))
+        pipe.sadd(_key("c:{0}:years".format(experiment_key)), date.strftime('%Y'))
+        pipe.sadd(_key("c:{0}:months".format(experiment_key)), date.strftime('%Y-%m'))
+        pipe.sadd(_key("c:{0}:days".format(experiment_key)), date.strftime('%Y-%m-%d'))
 
         pipe.execute()
 
         keys = [
-            _key("conversions:{0}:_all:users:all".format(experiment_key)),
-            _key("conversions:{0}:_all:users:{1}".format(experiment_key, date.strftime('%Y'))),
-            _key("conversions:{0}:_all:users:{1}".format(experiment_key, date.strftime('%Y-%m'))),
-            _key("conversions:{0}:_all:users:{1}".format(experiment_key, date.strftime('%Y-%m-%d'))),
-            _key("conversions:{0}:{1}:users:all".format(experiment_key, self.name)),
-            _key("conversions:{0}:{1}:users:{2}".format(experiment_key, self.name, date.strftime('%Y'))),
-            _key("conversions:{0}:{1}:users:{2}".format(experiment_key, self.name, date.strftime('%Y-%m'))),
-            _key("conversions:{0}:{1}:users:{2}".format(experiment_key, self.name, date.strftime('%Y-%m-%d'))),
+            _key("c:{0}:_all:users:all".format(experiment_key)),
+            _key("c:{0}:_all:users:{1}".format(experiment_key, date.strftime('%Y'))),
+            _key("c:{0}:_all:users:{1}".format(experiment_key, date.strftime('%Y-%m'))),
+            _key("c:{0}:_all:users:{1}".format(experiment_key, date.strftime('%Y-%m-%d'))),
+            _key("c:{0}:{1}:users:all".format(experiment_key, self.name)),
+            _key("c:{0}:{1}:users:{2}".format(experiment_key, self.name, date.strftime('%Y'))),
+            _key("c:{0}:{1}:users:{2}".format(experiment_key, self.name, date.strftime('%Y-%m'))),
+            _key("c:{0}:{1}:users:{2}".format(experiment_key, self.name, date.strftime('%Y-%m-%d'))),
         ]
-        msetbit(keys=keys, args=([client.sequential_id, 1] * len(keys)))
+        msetbit(keys=keys, args=([self.experiment.sequential_id(client), 1] * len(keys)))
 
     def conversion_rate(self):
         try:
@@ -578,12 +594,44 @@ class Alternative(object):
         except ZeroDivisionError:
             return 0
 
-    def z_score(self):
+    def g_stat(self):
+        # http://en.wikipedia.org/wiki/G-test
 
         if self.is_control():
             return 'N/A'
 
-        control = self.experiment.control()
+        control = self.experiment.control
+
+        alt_conversions = self.completed_count()
+        control_conversions = control.completed_count()
+        alt_failures = self.participant_count() - alt_conversions
+        control_failures = control.participant_count() - control_conversions
+
+        total_conversions = alt_conversions + control_conversions
+
+        if total_conversions < 20:
+            # small sample size of conversions, see where it goes for a bit
+            return 'N/A'
+
+        total_participants = self.participant_count() + control.participant_count()
+
+        expected_control_conversions = control.participant_count() * total_conversions / float(total_participants)
+        expected_alt_conversions = self.participant_count() * total_conversions / float(total_participants)
+        expected_control_failures = control.participant_count() - expected_control_conversions
+        expected_alt_failures = self.participant_count() - expected_alt_conversions
+
+        g_stat = 2 * (      alt_conversions * log(alt_conversions / expected_alt_conversions) \
+                        +   alt_failures * log(alt_failures / expected_alt_failures) \
+                        +   control_conversions * log(control_conversions / expected_control_conversions) \
+                        +   control_failures * log(control_failures / expected_control_failures))
+
+        return round(g_stat, 2)
+
+    def z_score(self):
+        if self.is_control():
+            return 'N/A'
+
+        control = self.experiment.control
         ctr_e = self.conversion_rate()
         ctr_c = control.conversion_rate()
 
@@ -591,13 +639,35 @@ class Alternative(object):
         c = control.participant_count()
 
         try:
-            std_dev = pow(((ctr_e / pow(ctr_c, 3)) * ((e*ctr_e)+(c*ctr_c)-(ctr_c*ctr_e)*(c+e))/(c*e)), 0.5)
+            std_dev = pow(((ctr_e / pow(ctr_c, 3)) * ((e * ctr_e) + (c * ctr_c) - (ctr_c * ctr_e) * (c + e)) / (c * e)), 0.5)
             z_score = ((ctr_e / ctr_c) - 1) / std_dev
             return z_score
         except ZeroDivisionError:
             return 0
 
-    def confidence_level(self):
+    def g_confidence_level(self):
+        # g stat is approximated by chi-square, we will use
+        # critical values from chi-square distribution with one degree of freedom
+
+        g_stat = self.g_stat()
+        if g_stat == 'N/A':
+            return g_stat
+
+        ret = ''
+        if g_stat == 0.0:
+            ret = 'No Change'
+        elif g_stat < 3.841:
+            ret = 'No Confidence'
+        elif g_stat < 6.635:
+            ret = '95% Confidence'
+        elif g_stat < 10.83:
+            ret = '99% Confidence'
+        else:
+            ret = '99.9% Confidence'
+
+        return ret
+
+    def z_confidence_level(self):
         z_score = self.z_score()
         if z_score == 'N/A':
             return z_score
@@ -618,6 +688,12 @@ class Alternative(object):
 
         return ret
 
+    def confidence_level(self, conf_type="g"):
+        if conf_type == "z":
+            return self.z_confidence_level()
+        else:
+            return self.g_confidence_level()
+
     def key(self):
         return _key("{0}:{1}".format(self.experiment.name, self.name))
 
