@@ -7,6 +7,7 @@ import redis
 from werkzeug.wrappers import Request, Response
 from werkzeug.routing import Map, Rule
 from werkzeug.exceptions import HTTPException, NotFound
+import dateutil.parser
 
 from . import __version__
 import decorator
@@ -86,7 +87,6 @@ class Sixpack(object):
             return json_success({'excluded': 'true'}, request)
 
         experiment_name = request.args.get('experiment')
-
         client_id = request.args.get('client_id')
 
         if client_id is None or experiment_name is None:
@@ -97,9 +97,12 @@ class Sixpack(object):
         try:
             experiment = Experiment.find(experiment_name, self.redis)
             if cfg.get('enabled', True):
-                alternative = experiment.convert(client)
+                dt = None
+                if request.args.get("datetime"):
+                    dt = dateutil.parser.parse(request.args.get("datetime"))
+                alternative = experiment.convert(client, dt=dt)
             else:
-                alternative = experiment.control().name
+                alternative = experiment.control.name
         except ValueError as e:
             return json_error({'message': str(e)}, request, 400)
 
@@ -129,21 +132,23 @@ class Sixpack(object):
         if client_id is None or experiment_name is None or alts is None:
             return json_error({'message': 'missing arguments'}, request, 400)
 
-        # Get the experiment ready for action
-        client = Client(client_id, self.redis)
         experiment = Experiment.find_or_create(experiment_name, alts, self.redis)
 
-        resp = {}
-
+        alternative = None
         if force and force in alts:
             alternative = force
-        elif not cfg.get('enabled', True) or should_exclude_visitor(request):
+        elif not cfg.get('enabled', True):
+            alternative = alts[0]
+        elif experiment.winner is not None:
+            alternative = experiment.winner
+        elif should_exclude_visitor(request):
             alternative = alts[0]
-            resp['excluded'] = True
-        elif experiment.has_winner():
-            alternative = experiment.get_winner().name
         else:
-            alternative = experiment.get_alternative(client).name
+            dt = None
+            if request.args.get("datetime"):
+                dt = dateutil.parser.parse(request.args.get("datetime"))
+            client = Client(client_id, self.redis)
+            alternative = experiment.get_alternative(client, dt=dt).name
 
         resp = {
             'alternative': {
@@ -168,11 +173,10 @@ def should_exclude_visitor(request):
 
 
 def is_robot(user_agent):
-    try:
-        regex = re.compile(r"{0}".format(cfg.get('robot_regex')), re.I)
-        return regex.match(unquote(user_agent))
-    except:
-        return False  # TODO Not sure if default should be true or false
+    if user_agent is None:
+        return False
+    regex = re.compile(r"{0}".format(cfg.get('robot_regex')), re.I)
+    return regex.search(unquote(user_agent)) is not None
 
 
 def is_ignored_ip(ip_address):
@@ -213,12 +217,10 @@ def _json_resp(in_dict, request, status=None):
 # Method to run with built-in server
 def create_app():
     app = Sixpack(db.REDIS)
-
     return app
 
 
 # Method to run with gunicorn
 def start(environ, start_response):
     app = Sixpack(db.REDIS)
-
     return app(environ, start_response)
