@@ -31,7 +31,7 @@ class TestExperimentModel(unittest.TestCase):
         pass
 
     def test_control(self):
-        control = self.exp_1.control()
+        control = self.exp_1.control
         self.assertEqual(control.name, 'yes')
 
     def test_created_at(self):
@@ -82,6 +82,17 @@ class TestExperimentModel(unittest.TestCase):
         with self.assertRaises(ValueError):
             Experiment.find('delete-me', self.redis)
 
+    def test_leaky_delete(self):
+        exp = Experiment('delete-me-1', self.alternatives, self.redis)
+        exp.save()
+
+        exp2 = Experiment('delete', self.alternatives, self.redis)
+        exp2.save()
+
+        exp2.delete()
+        exp3 = Experiment.find('delete-me-1', self.redis)
+        self.assertEqual(exp3.get_alternative_names(), self.alternatives)
+
     def test_archive(self):
         self.assertFalse(self.exp_1.is_archived())
         self.exp_1.archive()
@@ -105,38 +116,30 @@ class TestExperimentModel(unittest.TestCase):
         difference = new_version - original_version
         self.assertEqual(difference, 1)
 
-    def test_convert(self):
-        pass
-
     def test_set_winner(self):
         exp = Experiment('test-winner', ['1', '2'], self.redis)
         exp.set_winner('1')
-        self.assertTrue(exp.has_winner())
+        self.assertTrue(exp.winner is not None)
 
         exp.set_winner('1')
-        winner = exp.get_winner()
-        self.assertEqual(winner, '1')
+        self.assertEqual(exp.winner, '1')
 
-    def test_has_winner(self):
-        exp = Experiment('test-winner', ['1', '2'], self.redis)
-        self.assertFalse(exp.has_winner())
-
-    def test_get_winner(self):
+    def test_winner(self):
         exp = Experiment.find_or_create('test-get-winner', ['1', '2'], self.redis)
-        self.assertIsNone(exp.get_winner())
+        self.assertIsNone(exp.winner)
 
         exp.set_winner('1')
-        self.assertEqual(exp.get_winner(), '1')
+        self.assertEqual(exp.winner, '1')
 
     def test_reset_winner(self):
         exp = Experiment('show-something-reset-winner', self.alternatives, self.redis)
         exp.save()
         exp.set_winner('yes')
-        self.assertTrue(exp.has_winner())
-        self.assertEqual(exp.get_winner(), 'yes')
+        self.assertTrue(exp.winner is not None)
+        self.assertEqual(exp.winner, 'yes')
 
         exp.reset_winner()
-        self.assertFalse(exp.has_winner())
+        self.assertIsNone(exp.winner)
 
     def test_winner_key(self):
         exp = Experiment.find_or_create('winner-key', ['win', 'lose'], self.redis)
@@ -144,7 +147,6 @@ class TestExperimentModel(unittest.TestCase):
 
     def test_get_alternative(self):
         client = Client(10, self.redis)
-        client._sequential_id = 100
 
         exp = Experiment.find_or_create('archived-control', ['w', 'l'], self.redis)
         exp.archive()
@@ -167,7 +169,7 @@ class TestExperimentModel(unittest.TestCase):
         # most likely integration tests
 
     # See above note for the next 5 tests
-    def _test_get_alternative_by_client_id(self):
+    def _test_existing_alternative(self):
         pass
 
     def _test_has_converted_by_client(self):
@@ -188,15 +190,15 @@ class TestExperimentModel(unittest.TestCase):
 
     def test_key(self):
         key = self.exp_1.key()
-        self.assertEqual(key, 'sixpack:experiments:show-something-awesome/0')
+        self.assertEqual(key, 'sxp:e:show-something-awesome/0')
 
         key_2 = self.exp_2.key()
-        self.assertEqual(key_2, 'sixpack:experiments:dales-lagunitas/0')
+        self.assertEqual(key_2, 'sxp:e:dales-lagunitas/0')
 
         exp = Experiment('brews', ['mgd', 'bud-heavy'], self.redis)
         exp.increment_version()
         key_3 = exp.key()
-        self.assertEqual(key_3, 'sixpack:experiments:brews/1')
+        self.assertEqual(key_3, 'sxp:e:brews/1')
 
     def test_find(self):
         exp = Experiment('crunches-situps', ['crunches', 'situps'], self.redis)
