@@ -60,7 +60,10 @@ class TestServer(unittest.TestCase):
         data = json.loads(resp.data)
         self.assertEqual(200, resp.status_code)
         self.assertTrue('alternative' in data)
+        self.assertTrue('name' in data['alternative'])
         self.assertTrue('experiment' in data)
+        self.assertTrue('version' in data['experiment'])
+        self.assertTrue('name' in data['experiment'])
         self.assertTrue('client_id' in data)
         self.assertTrue('status' in data)
         self.assertEqual(data['status'], 'ok')
@@ -89,6 +92,14 @@ class TestServer(unittest.TestCase):
         self.assertEqual(200, resp.status_code)
         self.assertTrue('status' in data)
         self.assertEqual(data['status'], 'ok')
+        self.assertTrue('alternative' in data)
+        self.assertTrue('name' in data['alternative'])
+        self.assertTrue('experiment' in data)
+        self.assertTrue('version' in data['experiment'])
+        self.assertTrue('name' in data['experiment'])
+        self.assertTrue('client_id' in data)
+        self.assertTrue('conversion' in data)
+        self.assertTrue('value' in data['conversion'])
 
     def test_convert_fail(self):
         resp = self.client.get("/convert?experiment=baz&client_id=bar")
