@@ -1,7 +1,8 @@
 from flask import Flask
-from flask import render_template, abort, request, url_for, redirect, jsonify
+from flask import render_template, abort, request, url_for, redirect, jsonify, make_response
 from flask.ext.seasurf import SeaSurf
 from flask.ext.assets import Environment, Bundle
+from flask_debugtoolbar import DebugToolbarExtension
 
 from config import CONFIG as cfg
 from db import REDIS
@@ -11,13 +12,16 @@ import utils
 app = Flask(__name__)
 csrf = SeaSurf(app)
 
-js = Bundle('js/jquery.js', 'js/d3.js', 'js/bootstrap.js',
-            'js/bootstrap.min.js', 'js/chart.js',
-            'js/script.js', 'js/underscore-min.js',
+js = Bundle('js/jquery.js', 'js/d3.js',
+            'js/bootstrap.js', 'js/experiment.js', 'js/chart.js',
+            'js/script.js', 'js/underscore-min.js', 'js/spin.min.js',
+            'js/waypoints.min.js',
+            filters=['closure_js'],
             output="{0}/sixpack.js".format(cfg.get('asset_path', 'gen')))
 
 css = Bundle('css/bootstrap.css',
              'css/bootstrap-responsive.css', 'css/style.css',
+             filters=['yui_css'],
              output="{0}/sixpack.css".format(cfg.get('asset_path', 'gen')))
 
 assets = Environment(app)
@@ -28,12 +32,11 @@ assets.register('css_all', css)
 # List of experiments
 @app.route("/")
 def hello():
-    archived = bool(request.args.get('include_archived', False))
-    exclude_archived = not archived
-    experiments = Experiment.all(REDIS, exclude_archived)
-
-    return render_template('dashboard.html',
-                           experiments=experiments, include_archived=archived)
+    #archived = bool(request.args.get('include_archived', False))
+    #exclude_archived = not archived
+    # TODO: these need to be sorted
+    experiments = Experiment.all_names(REDIS)
+    return render_template('dashboard.html', experiments=experiments)
 
 
 # Details for experiment
@@ -45,7 +48,6 @@ def details(experiment_name):
 
 @app.route("/experiment/<experiment_name>.json")
 def json_details(experiment_name):
-
     period = request.args.get('period', None)
     if period not in ['day', 'week', 'month', 'year']:
         err = {'error': 'invalid argument: {0}'.format(period), 'status': 400}
@@ -56,6 +58,19 @@ def json_details(experiment_name):
     return jsonify(obj)
 
 
+@app.route("/experiment/<experiment_name>/export", methods=['POST'])
+def export(experiment_name):
+    experiment = find_or_404(experiment_name)
+
+    response = make_response(experiment.csvify())
+    response.headers["Content-Type"] = "text/csv"
+    # force a download with the content-disposition headers
+    filename = "sixpack_export_{0}".format(experiment_name)
+    response.headers["Content-Disposition"] = "attachment; filename={0}.csv".format(filename)
+
+    return response
+
+
 # Set winner for an experiment
 @app.route("/experiment/<experiment_name>/winner/", methods=['POST'])
 def set_winner(experiment_name):
@@ -138,6 +153,7 @@ def find_or_404(experiment_name):
 app.secret_key = cfg.get('secret_key')
 app.jinja_env.filters['number_to_percent'] = utils.number_to_percent
 app.jinja_env.filters['number_format'] = utils.number_format
+toolbar = DebugToolbarExtension(app)
 
 
 def start(environ, start_response):
